@@ -16,17 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-certificates-extension.c,v 1.11 2004/09/09 23:32:27 adamh Exp $
+ *  $Id: ephy-certificates-extension.c,v 1.20 2005/01/13 16:15:46 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-certificates-extension.h"
 
-#include "mozilla-embed-certificate.h"
-
 #include "ephy-debug.h"
 
 #include <epiphany/ephy-window.h>
@@ -174,7 +170,7 @@ view_certificate_cb (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	mozilla_embed_view_certificate (embed);
+	ephy_embed_show_page_certificate (embed);
 }
 
 static void
@@ -190,12 +186,12 @@ sync_security_status (EphyTab *tab,
 
 	if (ephy_window_get_active_tab (window) != tab) return;
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	action = gtk_ui_manager_get_action (manager, "/menubar/ViewMenu/ViewServerCertificateItem");
 	g_return_if_fail (action != NULL);
 
-	is_secure = ephy_tab_get_security_level (tab) > STATE_IS_INSECURE;
+	is_secure = ephy_tab_get_security_level (tab) > EPHY_EMBED_STATE_IS_INSECURE;
 	g_object_set (action, "sensitive", is_secure, NULL);
 }
 
@@ -211,8 +207,6 @@ tab_added_cb (GtkWidget *notebook,
 	embed = ephy_tab_get_embed (tab);	
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	mozilla_embed_certificate_attach (embed);
-
 	g_signal_connect_after (tab, "notify::security-level",
 				G_CALLBACK (sync_security_status), window);
 }
@@ -229,8 +223,6 @@ tab_removed_cb (GtkWidget *notebook,
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	mozilla_embed_certificate_detach (embed);
-
 	g_signal_handlers_disconnect_by_func
 		(tab, G_CALLBACK (sync_security_status), window);
 }
@@ -246,7 +238,7 @@ sync_active_tab_cb (EphyWindow *window,
 	sync_security_status (tab, NULL, window);
 }
 
-static void
+static gboolean
 padlock_button_press_cb (GtkWidget *ebox,
 			 GdkEventButton *event,
 			 EphyWindow *window)
@@ -256,10 +248,14 @@ padlock_button_press_cb (GtkWidget *ebox,
 	if (event->type == GDK_BUTTON_PRESS && event->button == 1 /* left */)
 	{
 		embed = ephy_window_get_active_embed (window);
-		g_return_if_fail (EPHY_IS_EMBED (embed));
+		g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
 	
-		mozilla_embed_view_certificate (embed);
+		ephy_embed_show_page_certificate (embed);
+
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 static GtkActionEntry action_entries_1 [] =
@@ -302,7 +298,7 @@ impl_attach_window (EphyExtension *ext,
 	guint ui_id;
 	WindowData *win_data;
 	GtkWidget *notebook;
-	GtkWidget *statusbar, *frame, *ebox;
+	GtkWidget *statusbar, *ebox;
 	GList *tabs, *l;
 
 	LOG ("EphyCertificatesExtension attach_window")
@@ -317,6 +313,8 @@ impl_attach_window (EphyExtension *ext,
 		tab_added_cb (notebook, l->data, window);
 	}
 
+	g_list_free (tabs);
+
 	g_signal_connect_after (notebook, "tab_added",
 				G_CALLBACK (tab_added_cb), window);
 	g_signal_connect_after (notebook, "tab_removed",
@@ -335,7 +333,7 @@ impl_attach_window (EphyExtension *ext,
 	/* add UI */
 	win_data = g_new (WindowData, 1);
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	win_data->action_group = action_group = gtk_action_group_new
 		("CertificatesExtensionActions");
@@ -357,10 +355,16 @@ impl_attach_window (EphyExtension *ext,
 			       "ViewServerCertificateItem",
 			       "ViewServerCertificate",
 			       GTK_UI_MANAGER_MENUITEM, FALSE);
+	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
+			       "ToolsSCSep1", NULL,
+			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
 			       "ToolsCertificateManagerItem",
 			       "ToolsCertificateManager",
 			       GTK_UI_MANAGER_MENUITEM, FALSE);
+	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
+			       "ToolsSCSep2", NULL,
+			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 
 	g_object_set_data_full (G_OBJECT (window), WINDOW_DATA_KEY, win_data,
 				(GDestroyNotify) free_window_data);
@@ -395,11 +399,11 @@ impl_detach_window (EphyExtension *ext,
 		(window, G_CALLBACK (sync_active_tab_cb), ext);
 
 	tabs = ephy_window_get_tabs (window);
-
 	for (l = tabs; l != NULL; l = g_list_next (l))
 	{
 		tab_removed_cb (notebook, l->data, window);
 	}
+	g_list_free (tabs);
 
 	/* un-make padlock icon clickable */
 	statusbar = ephy_window_get_statusbar (window);
@@ -408,7 +412,7 @@ impl_detach_window (EphyExtension *ext,
 		(ebox, G_CALLBACK (padlock_button_press_cb), window);
 
 	/* remove UI */
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	win_data = (WindowData *) g_object_get_data (G_OBJECT (window),
 						     WINDOW_DATA_KEY);
