@@ -17,12 +17,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-error-viewer-extension.c,v 1.16.2.2 2004/09/26 16:22:39 chpe Exp $
+ *  $Id: ephy-error-viewer-extension.c,v 1.21 2005/01/13 16:15:47 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "register-listener.h"
 #include "ephy-error-viewer-extension.h"
@@ -40,6 +38,7 @@
 #include <gtk/gtkactiongroup.h>
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtknotebook.h>
 
 #include <glib/gi18n-lib.h>
 
@@ -283,9 +282,9 @@ update_actions (EphyWindow *window)
 	g_value_init (&sensitive, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&sensitive, FALSE);
 
-	action1 = gtk_ui_manager_get_action (GTK_UI_MANAGER (window->ui_merge),
+	action1 = gtk_ui_manager_get_action (GTK_UI_MANAGER (ephy_window_get_ui_manager (window)),
 					     "/menubar/ToolsMenu/SgmlValidate");
-	action2 = gtk_ui_manager_get_action (GTK_UI_MANAGER (window->ui_merge),
+	action2 = gtk_ui_manager_get_action (GTK_UI_MANAGER (ephy_window_get_ui_manager (window)),
 					     "/menubar/ToolsMenu/CheckLinks");
 
 	tab = ephy_window_get_active_tab (window);
@@ -394,7 +393,7 @@ impl_attach_window (EphyExtension *extension,
 
 	data = g_new (WindowData, 1);
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	data->action_group = action_group =
 		gtk_action_group_new ("EphyErrorViewerExtensionActions");
@@ -449,7 +448,7 @@ impl_detach_window (EphyExtension *extension,
 #endif /* HAVE_OPENSP */
 
 	/* Remove UI */
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
 	g_return_if_fail (data != NULL);
