@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: extensions-manager-ui.c,v 1.10 2004/12/17 19:10:19 adamh Exp $
+ *  $Id: extensions-manager-ui.c,v 1.11 2005/01/13 16:15:48 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "extensions-manager-ui.h"
 #include "ephy-debug.h"
