@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-page-info-extension.c,v 1.8 2004/09/12 16:46:04 chpe Exp $
+ *  $Id: ephy-page-info-extension.c,v 1.15 2005/01/23 13:47:29 jframeau Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-page-info-extension.h"
 #include "ephy-debug.h"
@@ -29,6 +27,7 @@
 #include <epiphany/ephy-extension.h>
 #include <epiphany/ephy-window.h>
 #include <epiphany/ephy-embed.h>
+#include <epiphany/ephy-notebook.h>
 
 #include <gtk/gtkaction.h>
 #include <gtk/gtkactiongroup.h>
@@ -38,6 +37,7 @@
 #include <glib/gi18n-lib.h>
 
 #define WINDOW_DATA_KEY "EphyPageInfoExtensionWindowData"
+#define MENU_PATH "/menubar/ToolsMenu"
 
 #define EPHY_PAGE_INFO_EXTENSION_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_PAGE_INFO_EXTENSION, EphyPageInfoExtensionPrivate))
 
@@ -125,7 +125,7 @@ ephy_page_info_extension_init (EphyPageInfoExtension *extension)
 static void
 ephy_page_info_extension_finalize (GObject *object)
 {
-	EphyPageInfoExtension *extension = EPHY_PAGE_INFO_EXTENSION (object);
+//	EphyPageInfoExtension *extension = EPHY_PAGE_INFO_EXTENSION (object);
 
 	LOG ("EphyPageInfoExtension finalizing")
 
@@ -168,8 +168,8 @@ update_action (EphyWindow *window,
 	GtkAction *action;
 	gboolean loading = TRUE;
 
-	action = gtk_ui_manager_get_action (GTK_UI_MANAGER (window->ui_merge),
-					    "/menubar/ToolsMenu/PageInfo");
+	action = gtk_ui_manager_get_action (GTK_UI_MANAGER (ephy_window_get_ui_manager (window)),
+					    MENU_PATH "/PageInfo");
 
 	g_object_get (G_OBJECT (tab), "load-status", &loading, NULL);
 	g_object_set (G_OBJECT (action), "sensitive", !loading, NULL);
@@ -198,18 +198,18 @@ switch_page_cb (GtkNotebook *notebook,
 }
 
 static void
-tab_added_cb (GtkWidget *notebook,
-	      EphyTab *tab,
-	      EphyWindow *window)
+impl_attach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
 {
 	g_signal_connect_after (tab, "notify::load-status",
 				G_CALLBACK (load_status_cb), window);
 }
 
 static void
-tab_removed_cb (GtkWidget *notebook,
-		EphyTab *tab,
-		EphyWindow *window)
+impl_detach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
 {
 	g_signal_handlers_disconnect_by_func
 		(tab, G_CALLBACK (load_status_cb), window);
@@ -233,13 +233,12 @@ impl_attach_window (EphyExtension *extension,
 	guint ui_id;
 	WindowData *data;
 	GtkWidget *notebook;
-	GList *tabs, *l;
 
 	LOG ("EphyPageInfoExtension attach_window")
 
 	data = g_new (WindowData, 1);
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	data->action_group = action_group =
 		gtk_action_group_new ("EphyPageInfoExtensionActions");
@@ -254,29 +253,18 @@ impl_attach_window (EphyExtension *extension,
 	g_object_set_data_full (G_OBJECT (window), WINDOW_DATA_KEY, data,
 				(GDestroyNotify) free_window_data);
 
-	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
+	gtk_ui_manager_add_ui (manager, ui_id, MENU_PATH,
 			       "PageInfoSep", NULL,
 			       GTK_UI_MANAGER_SEPARATOR, FALSE);
-	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
+	gtk_ui_manager_add_ui (manager, ui_id, MENU_PATH,
 			       "PageInfo", "PageInfo",
 			       GTK_UI_MANAGER_MENUITEM, FALSE);
-	gtk_ui_manager_add_ui (manager, ui_id, "/menubar/ToolsMenu",
+	gtk_ui_manager_add_ui (manager, ui_id, MENU_PATH,
 			       "PageInfoSep2", NULL,
 			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 
 	notebook = ephy_window_get_notebook (window);
 
-	tabs = ephy_window_get_tabs (window);
-	for (l = tabs; l != NULL; l = l->next)
-	{
-		tab_added_cb (notebook, l->data, window);
-	}
-	g_list_free (tabs);
-
-	g_signal_connect_after (notebook, "tab_added",
-				G_CALLBACK (tab_added_cb), window);
-	g_signal_connect_after (notebook, "tab_removed",
-				G_CALLBACK (tab_removed_cb), window);
 	g_signal_connect_after (notebook, "switch_page",
 				G_CALLBACK (switch_page_cb), window);
 }
@@ -288,9 +276,8 @@ impl_detach_window (EphyExtension *extension,
 	GtkUIManager *manager;
 	WindowData *data;
 	GtkWidget *notebook;
-	GList *tabs, *l;
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	data = (WindowData *) g_object_get_data (G_OBJECT (window), WINDOW_DATA_KEY);
 	g_return_if_fail (data != NULL);
@@ -302,19 +289,8 @@ impl_detach_window (EphyExtension *extension,
 
 	notebook = ephy_window_get_notebook (window);
 
-	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_added_cb), window);
-	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_removed_cb), window);
 	g_signal_handlers_disconnect_by_func
 		(notebook, G_CALLBACK (switch_page_cb), window);
-
-	tabs = ephy_window_get_tabs (window);
-	for (l = tabs; l != NULL; l = l->next)
-	{
-		tab_removed_cb (notebook, l->data, window);
-	}
-	g_list_free (tabs);
 }
 
 static void
@@ -322,4 +298,6 @@ ephy_page_info_extension_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
+	iface->attach_tab = impl_attach_tab;
+	iface->detach_tab = impl_detach_tab;
 }
