@@ -19,7 +19,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-helpers.cpp,v 1.18 2004/12/18 04:59:05 adamh Exp $
+ *  $Id: mozilla-helpers.cpp,v 1.23 2005/01/13 21:43:46 jframeau Exp $
  */
 
 #include "mozilla-config.h"
@@ -251,13 +251,18 @@ PageInfoHelper::GetCacheEntryDescriptor (const nsAString &aUrl,
 {
   nsresult rv;
 
+  *aEntry = nsnull;
+ 
   nsCOMPtr<nsICacheService> cacheService =
           do_GetService(NS_CACHESERVICE_CONTRACTID);
   NS_ENSURE_TRUE (cacheService, NS_ERROR_FAILURE);
 
   nsEmbedCString cUrl;
   NS_UTF16ToCString (aUrl, NS_CSTRING_ENCODING_UTF8, cUrl);
-  
+
+  char *url = g_strdup (cUrl.get ());
+  g_strdelimit (url, "#", '\0'); /* snip fragment, see bug #161201 */
+
   const char *cacheTypes[] = { "HTTP", "FTP" };
   for (unsigned int i = 0 ; i < G_N_ELEMENTS (cacheTypes); i++)
     {
@@ -272,14 +277,20 @@ PageInfoHelper::GetCacheEntryDescriptor (const nsAString &aUrl,
   
       nsCOMPtr<nsICacheEntryDescriptor> cacheEntryDescriptor;
   
-      rv = cacheSession->OpenCacheEntry (cUrl.get(), nsICache::ACCESS_READ,
+#ifdef MOZ_NSICACHESESSION_OPENCACHEENTRY_NSACSTRING_
+      rv = cacheSession->OpenCacheEntry (nsEmbedCString(url), nsICache::ACCESS_READ,
                                          PR_FALSE, aEntry);
-  
-      if (NS_SUCCEEDED (rv)) return NS_OK;
+#else
+      rv = cacheSession->OpenCacheEntry (url, nsICache::ACCESS_READ,
+                                         PR_FALSE, aEntry);
+#endif
+
+      if (NS_SUCCEEDED (rv)) break;
     }
 
-  *aEntry = NULL;
-  return NS_ERROR_FAILURE;
+  g_free (url);
+
+  return rv;
 }
 
 nsresult
@@ -977,7 +988,7 @@ free_embed_page_form (EmbedPageForm *form)
   g_free (form);
 }
 
-extern "C" void
+static void
 mozilla_free_page_properties (EmbedPageProperties *props)
 {
   g_free (props->content_type);
@@ -998,6 +1009,8 @@ mozilla_free_embed_page_info (EmbedPageInfo *info)
   g_list_foreach (info->media, (GFunc) free_embed_page_medium, NULL);
   g_list_free (info->media);
 
+  mozilla_free_page_properties (info->props);
+
   g_free (info);
 }
 
