@@ -22,23 +22,24 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: page-info-dialog.c,v 1.20.2.4 2004/11/07 20:47:20 chpe Exp $
+ *  $Id: page-info-dialog.c,v 1.38 2005/01/13 16:15:48 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "page-info-dialog.h"
 #include "ephy-debug.h"
 
 #include <epiphany/ephy-embed-persist.h>
 #include <epiphany/ephy-embed-factory.h>
+#include <epiphany/ephy-state.h>
+#include <ephy-dnd.h>
 
 /* non-installed ephy headers */
-#include "ephy-state.h"
+#include "ephy-gui.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
+#include "ephy-file-chooser.h"
 
 #include <gtk/gtkentry.h>
 #include <gtk/gtklabel.h>
@@ -70,12 +71,12 @@ void page_info_dialog_close_button_clicked_cb	  (GtkWidget *button,
 						   PageInfoDialog *dialog);
 void page_info_dialog_view_cert_button_clicked_cb (GtkWidget *button,
 						   PageInfoDialog *dialog);
-void page_info_image_box_realize_cb		  (GtkContainer *box,
+void page_info_media_box_realize_cb		  (GtkContainer *box,
 						   PageInfoDialog *dialog);
 
 enum {
 	GENERAL_PAGE,
-	IMAGES_PAGE,
+	MEDIA_PAGE,
 	LINKS_PAGE,
 	FORMS_PAGE,
 	/*
@@ -110,6 +111,7 @@ struct _PageInfoDialogPrivate
 	EphyEmbed *embed;
 	GtkUIManager *manager;
 	GtkActionGroup *action_group;
+	EmbedPageInfo *page_info;
 };
 
 enum
@@ -142,10 +144,10 @@ enum
 
 	PROP_LINKS_LINK_TREEVIEW,
 
-	PROP_IMAGES_IMAGE_TREEVIEW,
-	PROP_IMAGES_IMAGE_BOX,
-	PROP_IMAGES_IMAGE_VPANED,
-	PROP_IMAGES_SAVE_BUTTON,
+	PROP_MEDIA_MEDIUM_TREEVIEW,
+	PROP_MEDIA_MEDIUM_BOX,
+	PROP_MEDIA_MEDIUM_VPANED,
+	PROP_MEDIA_SAVE_BUTTON,
 
 /*
 	PROP_SECURITY_CERT_TITLE,
@@ -179,10 +181,10 @@ EphyDialogProperty properties [] =
 
 	{ "page_info_link_list",	NULL, PT_NORMAL, 0 },
 
-	{ "page_info_image_list",	NULL, PT_NORMAL, 0 },
-	{ "page_info_image_box",	NULL, PT_NORMAL, 0 },
-	{ "page_info_image_vpaned",	NULL, PT_NORMAL, 0 },
-	{ "page_info_image_save",	NULL, PT_NORMAL, 0 },
+	{ "page_info_media_list",	NULL, PT_NORMAL, 0 },
+	{ "page_info_media_box",	NULL, PT_NORMAL, 0 },
+	{ "page_info_media_vpaned",	NULL, PT_NORMAL, 0 },
+	{ "page_info_media_save",	NULL, PT_NORMAL, 0 },
 
 	/*
 	{ PROP_SECURITY_CERT_TITLE,     "page_info_security_title", NULL, PT_NORMAL, NULL },
@@ -195,6 +197,12 @@ EphyDialogProperty properties [] =
 	{ NULL }
 };
 
+static GtkTargetEntry drag_targets[] =
+{
+	{ EPHY_DND_URL_TYPE, 0, 0 }
+};
+static int n_drag_targets = G_N_ELEMENTS (drag_targets);
+
 /*
 enum
 {
@@ -481,7 +489,9 @@ page_info_dialog_view_cert_button_clicked_cb (GtkWidget *button,
 /* a generic treeview info page */
 
 typedef struct _TreeviewInfoPage TreeviewInfoPage;
-	
+
+typedef void (* TreeviewInfoPageFilterFunc)  (TreeviewInfoPage *);
+
 struct _TreeviewInfoPage
 {
 	InfoPage page;
@@ -492,6 +502,7 @@ struct _TreeviewInfoPage
 	GtkActionEntry *action_entries;
 	guint n_action_entries;
 	const char *popup_path;
+        TreeviewInfoPageFilterFunc filter_func;
 };
 
 static gboolean
@@ -503,7 +514,8 @@ treeview_info_page_show_popup (TreeviewInfoPage *page)
 
 	widget = gtk_ui_manager_get_widget (dialog->priv->manager,
 					    page->popup_path);
-	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, 0,
+	gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
+			ephy_gui_menu_position_tree_selection, page->treeview, 0,
 			gtk_get_current_event_time ());
 	gtk_menu_shell_select_first (GTK_MENU_SHELL (widget), FALSE);
 
@@ -515,10 +527,13 @@ treeview_info_page_button_pressed_cb (GtkTreeView *treeview,
 				      GdkEventButton *event,
 				      TreeviewInfoPage *page)
 {
+	InfoPage *ipage = (InfoPage *) page;
+	PageInfoDialog *dialog = ipage->dialog;
 	GtkTreeModel *model = GTK_TREE_MODEL (page->store);
 	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	GtkTreePath *path = NULL;
+	GtkWidget *widget;
 
 	/* right-click? */
 	if (event->button != 3)
@@ -542,11 +557,25 @@ treeview_info_page_button_pressed_cb (GtkTreeView *treeview,
 
 	/* Select the row the user clicked on */
 	selection = gtk_tree_view_get_selection (treeview);
-	gtk_tree_selection_unselect_all (selection);
-	gtk_tree_selection_select_path (selection, path);
-	gtk_tree_path_free (path);
+	if (gtk_tree_selection_count_selected_rows (selection) == 1)
+	{
+		gtk_tree_selection_unselect_all (selection);
+		gtk_tree_selection_select_path (selection, path);
+		gtk_tree_path_free (path);
+	}
 
-	return treeview_info_page_show_popup (page);
+        if (page->filter_func)
+        {
+		page->filter_func (page);
+        }
+       
+	/* now popup the menu */
+	widget = gtk_ui_manager_get_widget (dialog->priv->manager,
+					    page->popup_path);
+	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
+			event->button, event->time);
+
+	return TRUE;
 }
 
 static void
@@ -579,18 +608,7 @@ page_info_set_text (PageInfoDialog *dialog,
 	/* FIXME: Switch to prop strings instead of enum */
 	widget = ephy_dialog_get_control (EPHY_DIALOG (dialog), prop);
 
-	g_return_if_fail (GTK_IS_ENTRY (widget) || GTK_IS_LABEL (widget));
-
-	if (!text) text = "";
-
-	if (GTK_IS_ENTRY(widget))
-	{
-		gtk_entry_set_text (GTK_ENTRY (widget), text);
-	}
-	else
-	{
-		gtk_label_set_markup (GTK_LABEL (widget), text);
-	}
+	gtk_label_set_text (GTK_LABEL (widget), text ? text : "");
 }
 
 static void
@@ -609,7 +627,7 @@ general_info_page_fill (InfoPage *page)
 
 	LOG ("general_info_page_fill")
 
-	props = mozilla_get_page_properties (embed);
+	props = dialog->priv->page_info->props;
 	g_return_if_fail (props != NULL);
 
 	val = ephy_embed_get_title (embed);
@@ -677,7 +695,7 @@ general_info_page_fill (InfoPage *page)
 
 	page_info_set_text (dialog, properties[PROP_GENERAL_REFERRING_URL].id,
 			    props->referring_url ? props->referring_url :
-			    _("No referrer" ));
+			    _("No referrer"));
 	
 	if (props->modification_time)
 	{
@@ -719,8 +737,6 @@ general_info_page_fill (InfoPage *page)
 						(EmbedPageMetaTag*)i->data);
 	}	
 	*/
-
-	mozilla_free_page_properties (props);
 }
 
 static InfoPage *
@@ -734,15 +750,24 @@ general_info_page_new (PageInfoDialog *dialog)
 	return ipage;
 }
 
-/* "Images" page */
+/* Set dnd cursor to the default dnd Gtk one and not the GtktreeView one */
+static void 
+page_info_drag_begin_cb(GtkWidget *widget, 
+		        GdkDragContext *dc, 
+		        gpointer dummy)
+{
+	gtk_drag_set_icon_default (dc);
+}
 
-#define IMAGE_PANED_POSITION_DEFAULT	250
+/* "Media" page */
+
+#define MEDIUM_PANED_POSITION_DEFAULT	250
 #define CONF_DESKTOP_BG_PICTURE		"/desktop/gnome/background/picture_filename"
 #define CONF_DESKTOP_BG_TYPE		"/desktop/gnome/background/picture_options"
 
-typedef struct _ImagesInfoPage ImagesInfoPage;
+typedef struct _MediaInfoPage MediaInfoPage;
 
-struct _ImagesInfoPage
+struct _MediaInfoPage
 {
 	TreeviewInfoPage tpage;
 
@@ -752,62 +777,232 @@ struct _ImagesInfoPage
 
 enum
 {
-	COL_IMAGE_URL,
-	COL_IMAGE_ALT,
-	COL_IMAGE_TITLE,
-	COL_IMAGE_WIDTH,
-	COL_IMAGE_HEIGHT
+	COL_MEDIUM_URL,
+	COL_MEDIUM_TYPE,
+        COL_MEDIUM_TYPE_TEXT,
+	COL_MEDIUM_ALT,
+	COL_MEDIUM_TITLE,
+	COL_MEDIUM_WIDTH,
+	COL_MEDIUM_HEIGHT
 };
 
-static char *
-images_get_selected_image_url (ImagesInfoPage *page)
+enum
+{
+	TV_COL_MEDIUM_URL,
+        TV_COL_MEDIUM_TYPE_TEXT,
+	TV_COL_MEDIUM_ALT,
+	TV_COL_MEDIUM_TITLE,
+	TV_COL_MEDIUM_WIDTH,
+	TV_COL_MEDIUM_HEIGHT,
+};
+
+static const char *
+mozilla_embed_type_to_string (EmbedPageMediumType type)
+{
+	const char *s_type;
+
+	switch (type)
+	{
+		case MEDIUM_IMAGE: 
+			s_type = _("Image");
+			break;
+		case MEDIUM_EMBED:
+			s_type = _("Embed");
+			break;
+		case MEDIUM_OBJECT:
+			s_type = _("Object");
+			break;
+		case MEDIUM_APPLET:
+			s_type = _("Applet");
+			break;
+		case MEDIUM_ICON:
+			s_type = _("Icon");
+			break;
+		default:
+			s_type = _("Unknown");
+	}
+
+	return s_type;
+}
+
+static EmbedPageMediumType
+media_get_selected_medium_type (GtkTreeSelection *selection)
+{
+	GtkTreeModel *model;
+	GList *selected_row;
+	GtkTreeIter iter;
+	EmbedPageMediumType type = 0;
+
+	selected_row = gtk_tree_selection_get_selected_rows (selection, &model);
+
+	if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *)(selected_row->data)))
+	{
+		gtk_tree_model_get (model, &iter, COL_MEDIUM_TYPE, &type, -1);
+	}
+	g_list_free (selected_row);
+
+	return type;
+}
+
+static gboolean
+media_is_embedded_medium (EmbedPageMediumType type)
+{
+	return type == MEDIUM_OBJECT || type == MEDIUM_EMBED;
+}
+
+/* Return a list of selected addresses (char *) */
+static GList *
+media_get_selected_medium_url (MediaInfoPage *page)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	GtkTreeModel *model;
-	GtkTreeIter iter;
 	char *address = NULL;
+	GList *selected_rows;
+	GList *ret = NULL, *l;
 
-	if (gtk_tree_selection_get_selected (tpage->selection, &model, &iter))
+	selected_rows = gtk_tree_selection_get_selected_rows (tpage->selection, &model);
+
+	for (l=selected_rows; l != NULL; l = l->next)
 	{
-		gtk_tree_model_get (model, &iter,
-				    COL_IMAGE_URL, &address,
-				    -1);
+		GtkTreeIter iter;
+
+		if (gtk_tree_model_get_iter (model, &iter, 
+					     (GtkTreePath *)(l->data)))
+		{
+			gtk_tree_model_get (model, &iter, COL_MEDIUM_URL, &address, -1);
+
+			ret = g_list_prepend (ret, address);
+		}
 	}
 
-	return address;
+	g_list_free (selected_rows);
+
+	return ret;
+}
+
+/* Save a medium given the destination directory */
+static void
+save_a_medium (const char *source, const char *dir)
+{
+	GnomeVFSURI *uri;
+	EphyEmbedPersist *persist;
+
+	uri = gnome_vfs_uri_new (source);
+
+	if (uri) 
+	{	
+		char *file_name;
+
+		file_name = gnome_vfs_uri_extract_short_name (uri);
+
+		if (file_name != NULL)
+		{
+			char *dest;
+
+			/* Persist needs a full path */
+			dest = g_strconcat (dir, "/", file_name, NULL);
+
+			persist = EPHY_EMBED_PERSIST (
+					ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
+			ephy_embed_persist_set_source (persist, source);
+			ephy_embed_persist_set_dest   (persist, dest);
+			ephy_embed_persist_save (persist);
+
+			g_object_unref (persist);
+			gnome_vfs_uri_unref (uri);
+			g_free(dest);
+		}
+		g_free(file_name);
+	}
+}
+
+/* Download at the same time all the selelected medium (rows) */
+static void
+download_path_response_cb (GtkDialog *fc, gint response, GList *rows)
+{
+	if (response == GTK_RESPONSE_ACCEPT)
+	{
+		char *dir;
+		GList *l;
+
+		dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
+
+		if (dir)
+		{
+			for (l = rows; l != NULL; l = l->next)
+			{
+				save_a_medium ((char *)l->data, dir);
+			}
+		}
+		g_free (dir);
+	}
+	gtk_widget_destroy (GTK_WIDGET (fc));
+
+	/* No needs of rows anymore */
+	g_list_foreach (rows, (GFunc) g_free, NULL);
+	g_list_free (rows);
 }
 
 static void
-images_open_image_cb (GtkAction *action,
-		      ImagesInfoPage *page)
+media_open_medium_cb (GtkAction *action,
+		      MediaInfoPage *page)
 {
 	/* FIXME: write me! :) */
 }
 
+
 static void
-images_save_image_cb (gpointer ptr,
-		      ImagesInfoPage *page)
+media_save_medium_cb (gpointer ptr,
+		      MediaInfoPage *page)
 {
 	InfoPage *ipage = (InfoPage *) page;
 	PageInfoDialog *dialog = ipage->dialog;
 	EphyEmbedPersist *persist;
-	char *address;
+	GList *rows;
 
-	address = images_get_selected_image_url (page);
-	if (address == NULL) return;
+	rows = media_get_selected_medium_url (page);
+	if (g_list_length (rows) == 1)
+	{
+		char *address = (gchar *)(rows->data); 
 
-	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		if (address != NULL)
+		{
+		persist = EPHY_EMBED_PERSIST
+					(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
-	ephy_embed_persist_set_source (persist, address);
-	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_ASK_DESTINATION);
-	ephy_embed_persist_set_fc_title (persist, _("Save Image As..."));
-	ephy_embed_persist_set_fc_parent (persist, GTK_WINDOW (dialog->priv->window));
+		ephy_embed_persist_set_source (persist, address);
+		ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_ASK_DESTINATION);
+		ephy_embed_persist_set_fc_title (persist, _("Save Medium As..."));
+		ephy_embed_persist_set_fc_parent (persist, GTK_WINDOW (dialog->priv->window));
 
-	ephy_embed_persist_save (persist);
+		ephy_embed_persist_save (persist);
 
-	g_object_unref (persist);
-	g_free (address);
+		g_object_unref (persist);
+		}
+		g_list_foreach (rows, (GFunc) g_free, NULL);
+		g_list_free (rows);
+	}
+	else
+	{
+		EphyFileChooser *fc;
+		GtkWidget *parent;
+
+		parent = NULL;
+
+		fc = ephy_file_chooser_new (_("Select a directory"),
+				            GTK_WIDGET (parent),
+				            GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+				            NULL, EPHY_FILE_FILTER_NONE);
+
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fc),
+						     g_get_home_dir ());
+
+		g_signal_connect (GTK_DIALOG (fc), "response",
+				  G_CALLBACK (download_path_response_cb),
+				  rows);
+
+		gtk_widget_show (GTK_WIDGET (fc));
+	}
 }
 
 static void
@@ -828,26 +1023,30 @@ background_download_completed_cb (EphyEmbedPersist *persist)
 }
 
 static void
-images_set_image_as_background_cb (GtkAction *action,
-				   ImagesInfoPage *page)
+media_set_image_as_background_cb (GtkAction *action,
+				  MediaInfoPage *page)
 {
 	EphyEmbedPersist *persist;
 	char *address;
 	char *dest, *base, *base_converted;
+	GList *rows;
 
-	address = images_get_selected_image_url (page);
-	if (address == NULL) return;
+	rows = media_get_selected_medium_url (page);
+	g_return_if_fail (g_list_length (rows) == 1);
 
+	address = (gchar *)(rows->data); 
+	if (address != NULL)
+	{
 	base = g_path_get_basename (address);
 	base_converted = g_filename_from_utf8 (base, -1, NULL, NULL, NULL);
 	dest = g_build_filename (ephy_dot_dir (), base_converted, NULL);
 
 	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_source (persist, address);
 	ephy_embed_persist_set_dest (persist, dest);
-	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW);
 
 	g_signal_connect (persist, "completed",
 			  G_CALLBACK (background_download_completed_cb), NULL);
@@ -856,109 +1055,196 @@ images_set_image_as_background_cb (GtkAction *action,
 
 	g_object_unref (persist);
 
-	g_free (address);
 	g_free (dest);
 	g_free (base);
 	g_free (base_converted);
+	}
+	g_list_foreach (rows, (GFunc) g_free, NULL);
+	g_list_free (rows);
 }
 
 static void
-images_copy_image_address_cb (GtkAction *action,
-			      ImagesInfoPage *page)
+media_copy_medium_address_cb (GtkAction *action,
+			      MediaInfoPage *page)
 {
 	char *address;
+	GList *rows;
 
-	address = images_get_selected_image_url (page);
-	if (address == NULL) return;
+	rows = media_get_selected_medium_url (page);
+	g_return_if_fail (g_list_length (rows) == 1);
 
+	address = (gchar *)(rows->data);
+	g_return_if_fail (address != NULL);
+	if (address != NULL)
+	{
 	gtk_clipboard_set_text (gtk_clipboard_get (GDK_NONE),
 				address, -1);
 	gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
 				address, -1);
-
-	g_free (address);
+	}
+	g_list_foreach (rows, (GFunc) g_free, NULL);
+	g_list_free (rows);
 }
 
-static GtkActionEntry images_action_entries[] =
+static GtkActionEntry media_action_entries[] =
 {
-	{ "OpenImage",
+	{ "Open",
 	  GTK_STOCK_OPEN,
-	  N_("_Open Image"),
+	  N_("_Open"),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (images_open_image_cb) },
-	{ "CopyImageAddress", 
+	  G_CALLBACK (media_open_medium_cb) },
+	{ "CopyMediumAddress",
 	  GTK_STOCK_COPY,
-	  N_("_Copy Image Address"),
+	  N_("_Copy Link Address"),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (images_copy_image_address_cb) },
-	{ "SetImageAsBackground",
+	  G_CALLBACK (media_copy_medium_address_cb) },
+	{ "SetAsBackground",
 	  NULL,
-	  N_("_Use Image as Background"),
+	  N_("_Use as Background"),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (images_set_image_as_background_cb) },
+	  G_CALLBACK (media_set_image_as_background_cb) },
 	{ "SaveAs", 
 	  GTK_STOCK_SAVE_AS,
-	  N_("_Save Image As..."),
+	  N_("_Save As..."),
 	  NULL,
 	  NULL,
-	  G_CALLBACK (images_save_image_cb) }
+	  G_CALLBACK (media_save_medium_cb) }
 };
 
 void
-page_info_image_box_realize_cb (GtkContainer *box,
+page_info_media_box_realize_cb (GtkContainer *box,
 				PageInfoDialog *dialog)
 {
-	TreeviewInfoPage *tpage = (TreeviewInfoPage *) dialog->priv->pages[IMAGES_PAGE];
-	ImagesInfoPage *page = (ImagesInfoPage *) tpage;
+	TreeviewInfoPage *tpage = (TreeviewInfoPage *) dialog->priv->pages[MEDIA_PAGE];
+	MediaInfoPage *page = (MediaInfoPage *) tpage;
 	EphyEmbed *embed;
 
-	page->embed = embed = EPHY_EMBED (ephy_embed_factory_new_object ("EphyEmbed"));
+	page->embed = embed = EPHY_EMBED (ephy_embed_factory_new_object (EPHY_TYPE_EMBED));
 
 	/* FIXME: this doesn't seem to work!? */
-	/* When the image has loaded grab the focus for the treeview
+	/* When the media has loaded grab the focus for the treeview
 	 * again. This means that you can navigate in the treeview
 	 * using the arrow keys */
 	g_signal_connect_swapped (embed, "net_stop",
 				  G_CALLBACK (gtk_widget_grab_focus),
 				  tpage->treeview);
 
-	ephy_embed_load_url (embed, "about:blank");
-
 	gtk_widget_show (GTK_WIDGET(embed));
 
 	gtk_container_add (box, GTK_WIDGET (embed));
+
+	ephy_embed_load_url (embed, "about:blank");
 }
 
-static void
-images_treeview_selection_changed_cb (GtkTreeSelection *selection,
-				      ImagesInfoPage *page)
+static gboolean 
+media_treeview_selection_changed_cb (GtkTreeSelection *selection,
+                                     GtkTreeModel *model,
+                                     GtkTreePath *path,
+                                     gboolean path_currently_selected,
+                                     MediaInfoPage *page)
 {
-	char *url;
+	if (path_currently_selected) return TRUE;
 
-	url = images_get_selected_image_url (page);
-
-	if (url != NULL)
+	/* Display medium if only one selected and medium is not an embed */
+	if (gtk_tree_selection_count_selected_rows (selection) == 0)
 	{
-		ephy_embed_load_url (page->embed, url);
+		GtkTreeIter iter;
+		EmbedPageMediumType type;
+		char *url;
+
+		if (gtk_tree_model_get_iter (model, &iter, path))
+		{
+			gtk_tree_model_get (model, &iter,
+					    COL_MEDIUM_URL, &url,
+					    COL_MEDIUM_TYPE, &type,
+					    -1);
+
+		}
+
+		if (url != NULL && !media_is_embedded_medium (type))
+		{
+			ephy_embed_load_url (page->embed, url);
+		}
+		else
+		{
+			ephy_embed_load_url (page->embed, "about:blank");
+		}
+		gtk_widget_set_sensitive (page->save_button, url != NULL);
+
+		g_free (url);
 	}
 	else
 	{
 		ephy_embed_load_url (page->embed, "about:blank");
 	}
-
-	gtk_widget_set_sensitive (page->save_button, url != NULL);
-
-	g_free (url);
+	return TRUE;
 }
 
 static void
-images_info_page_construct (InfoPage *ipage)
+media_info_page_filter (TreeviewInfoPage *page)
+{
+	InfoPage *ipage = (InfoPage *) page;
+	PageInfoDialog *dialog = ipage->dialog;
+	GtkAction *action;
+	gboolean one_col, can_set_as_background = FALSE;
+	EmbedPageMediumType type = MEDIUM_IMAGE;
+
+	/* In case of a multiple medium selection, we must reduce popup to Save As
+	 * only. Other point is that Embeds and Objects cannot be used as background!
+         */
+	one_col = (gtk_tree_selection_count_selected_rows (page->selection) == 1);
+
+	if (one_col)
+	{
+		type = media_get_selected_medium_type (page->selection);
+
+		can_set_as_background = ! media_is_embedded_medium (type);
+	}
+	action = gtk_action_group_get_action (dialog->priv->action_group,
+					      "CopyMediumAddress");
+	g_object_set (action, "visible", one_col, NULL);
+	action = gtk_action_group_get_action (dialog->priv->action_group,
+					      "SetAsBackground");
+	g_object_set (action, "visible", one_col && can_set_as_background, NULL);
+}
+
+static void
+media_drag_data_get_cb (GtkWidget *widget,
+                        GdkDragContext *context,
+                        GtkSelectionData *selection_data,
+                        guint info,
+                        guint32 time,
+                        MediaInfoPage *page)
+{
+	char *address;
+	GList *rows;
+
+	g_assert (widget != NULL);
+	g_return_if_fail (context != NULL);
+
+	rows = media_get_selected_medium_url (page);
+	g_return_if_fail (rows != NULL);
+
+	address = (gchar *)(rows->data);
+	g_return_if_fail (address != NULL);
+
+	gtk_selection_data_set (selection_data,
+			selection_data->target,
+			8,
+			address,
+			strlen(address));
+	g_list_foreach (rows, (GFunc) g_free, NULL);
+	g_list_free (rows);
+}
+
+static void
+media_info_page_construct (InfoPage *ipage)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
-	ImagesInfoPage *page = (ImagesInfoPage *) ipage;
+	MediaInfoPage *page = (MediaInfoPage *) ipage;
 	PageInfoDialog *dialog = ipage->dialog;
 	GtkTreeView *treeview;
 	GtkListStore *liststore;
@@ -969,95 +1255,124 @@ images_info_page_construct (InfoPage *ipage)
 	GtkAction *action;
 
 	treeview = GTK_TREE_VIEW (ephy_dialog_get_control
-		(EPHY_DIALOG (dialog), properties[PROP_IMAGES_IMAGE_TREEVIEW].id));
+		(EPHY_DIALOG (dialog), properties[PROP_MEDIA_MEDIUM_TREEVIEW].id));
 
-	liststore = gtk_list_store_new (5,
+	liststore = gtk_list_store_new (7,
+					G_TYPE_STRING,
+					G_TYPE_INT,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_INT,
-					G_TYPE_INT);
+                                        G_TYPE_INT);
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (liststore));
 	g_object_unref (liststore);
 
-	selection = gtk_tree_view_get_selection (treeview);
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+	/* Allow the tree to be a source for dnd */
+	gtk_tree_view_enable_model_drag_source (treeview, GDK_BUTTON1_MASK,
+                                                drag_targets, n_drag_targets,
+                                                GDK_ACTION_COPY);
+	g_signal_connect (G_OBJECT(treeview),
+                          "drag_data_get",
+                          G_CALLBACK(media_drag_data_get_cb),
+                          page);
 
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (images_treeview_selection_changed_cb),
-			  page);
+	/* Be careful, default handler forces dnd icon to be an image of the row ! */ 
+	g_signal_connect_after (G_OBJECT(treeview),
+                                "drag_begin",
+                                G_CALLBACK(page_info_drag_begin_cb),
+                                NULL);
+
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+	gtk_tree_selection_set_select_function (selection,
+						(GtkTreeSelectionFunc)
+							media_treeview_selection_changed_cb,
+						page, NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_IMAGE_URL,
+						     TV_COL_MEDIUM_URL,
 						     _("URL"),
 						     renderer,
-						     "text", COL_IMAGE_URL,
+						     "text", COL_MEDIUM_URL,
 						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_IMAGE_URL);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_URL);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_IMAGE_URL);
+	gtk_tree_view_column_set_sort_column_id (column, TV_COL_MEDIUM_URL);
 
 	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_IMAGE_ALT, 
+						     TV_COL_MEDIUM_TYPE_TEXT,
+						     _("Type"),
+						     renderer,
+						     "text", COL_MEDIUM_TYPE_TEXT,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_TYPE_TEXT);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_reorderable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_MEDIUM_TYPE_TEXT);
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     TV_COL_MEDIUM_ALT, 
 						     _("Alt Text"),
 						     renderer,
-						     "text", COL_IMAGE_ALT,
+						     "text", COL_MEDIUM_ALT,
 						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_IMAGE_ALT);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_ALT);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_IMAGE_ALT);
+	gtk_tree_view_column_set_sort_column_id (column, COL_MEDIUM_ALT);
 
 	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_IMAGE_TITLE, 
+						     TV_COL_MEDIUM_TITLE, 
 						     _("Title"),
 						     renderer,
-						     "text", COL_IMAGE_TITLE,
+						     "text", COL_MEDIUM_TITLE,
 						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_IMAGE_TITLE);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_TITLE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_IMAGE_TITLE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_MEDIUM_TITLE);
 
 	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_IMAGE_WIDTH, 
+						     TV_COL_MEDIUM_WIDTH, 
 						     _("Width"),
 						     renderer,
-						     "text", COL_IMAGE_WIDTH,
+						     "text", COL_MEDIUM_WIDTH,
 						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_IMAGE_WIDTH);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_WIDTH);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_IMAGE_WIDTH);
+	gtk_tree_view_column_set_sort_column_id (column, COL_MEDIUM_WIDTH);
 
 	gtk_tree_view_insert_column_with_attributes (treeview,
-						     COL_IMAGE_HEIGHT, 
+						     TV_COL_MEDIUM_HEIGHT, 
 						     _("Height"),
 						     renderer,
-						     "text", COL_IMAGE_HEIGHT,
+						     "text", COL_MEDIUM_HEIGHT,
 						     NULL);
-	column = gtk_tree_view_get_column (treeview, COL_IMAGE_HEIGHT);
+	column = gtk_tree_view_get_column (treeview, TV_COL_MEDIUM_HEIGHT);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COL_IMAGE_HEIGHT);
+	gtk_tree_view_column_set_sort_column_id (column, COL_MEDIUM_HEIGHT);
 
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog),
-					  properties[PROP_IMAGES_SAVE_BUTTON].id);
+					  properties[PROP_MEDIA_SAVE_BUTTON].id);
 	g_signal_connect (button, "clicked",
-			  G_CALLBACK (images_save_image_cb), page);
+			  G_CALLBACK (media_save_medium_cb), page);
 
 	vpaned = ephy_dialog_get_control (EPHY_DIALOG (dialog),
-					  properties[PROP_IMAGES_IMAGE_VPANED].id);
+					  properties[PROP_MEDIA_MEDIUM_VPANED].id);
 
-	ephy_state_add_paned (vpaned, "PageInfoDialog::ImagesPage::VPaned",
-			      IMAGE_PANED_POSITION_DEFAULT);
+	ephy_state_add_paned (vpaned, "PageInfoDialog::MediaPage::VPaned",
+			      MEDIUM_PANED_POSITION_DEFAULT);
 
 	tpage->store = liststore;
 	tpage->selection = selection;
@@ -1068,53 +1383,53 @@ images_info_page_construct (InfoPage *ipage)
 
 	/* FIXME: implement "Open", then remove this */
 	action = gtk_action_group_get_action (dialog->priv->action_group,
-					      "OpenImage");
+					      "Open");
 	g_object_set (action, "visible", FALSE, NULL);
 }
 
 static void
-images_info_page_fill (InfoPage *ipage)
+media_info_page_fill (InfoPage *ipage)
 {
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) ipage;
 	PageInfoDialog *dialog = ipage->dialog;
 	GtkListStore *store = tpage->store;
 	GtkTreeIter iter;
-	GList *images, *l;
+	GList *media, *l;
 
-	images = mozilla_get_images (dialog->priv->embed);
+	media = dialog->priv->page_info->media;
 
-	for (l = images; l != NULL; l = l->next)
+	for (l = media; l != NULL; l = l->next)
 	{
-		EmbedPageImage *image = (EmbedPageImage *) l->data;
+		EmbedPageMedium *media = (EmbedPageMedium *) l->data;
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
-				    COL_IMAGE_URL, image->url,
-				    COL_IMAGE_ALT, image->alt,
-				    COL_IMAGE_TITLE, image->title,
-				    COL_IMAGE_WIDTH, image->width,
-				    COL_IMAGE_HEIGHT, image->height,
+				    COL_MEDIUM_URL, media->url,
+				    COL_MEDIUM_TYPE_TEXT, mozilla_embed_type_to_string (media->type),
+				    COL_MEDIUM_ALT, media->alt,
+				    COL_MEDIUM_TITLE, media->title,
+				    COL_MEDIUM_WIDTH, media->width,
+				    COL_MEDIUM_HEIGHT, media->height,
+				    COL_MEDIUM_TYPE, media->type,
 				    -1);
 	}
-
-	g_list_foreach (images, (GFunc) mozilla_free_embed_page_image, NULL);
-	g_list_free (images);
 }
 
 static InfoPage *
-images_info_page_new (PageInfoDialog *dialog)
+media_info_page_new (PageInfoDialog *dialog)
 {
-	ImagesInfoPage *page = g_new0 (ImagesInfoPage, 1);
+	MediaInfoPage *page = g_new0 (MediaInfoPage, 1);
 	TreeviewInfoPage *tpage = (TreeviewInfoPage *) page;
 	InfoPage *ipage = (InfoPage *) page;
 
 	ipage->dialog = dialog;
-	ipage->construct = images_info_page_construct;
-	ipage->fill = images_info_page_fill;
+	ipage->construct = media_info_page_construct;
+	ipage->fill = media_info_page_fill;
 
-	tpage->popup_path = "/ImagesPopup";
-	tpage->action_entries = images_action_entries;
-	tpage->n_action_entries = G_N_ELEMENTS (images_action_entries);
+	tpage->popup_path = "/MediaPopup";
+	tpage->action_entries = media_action_entries;
+	tpage->n_action_entries = G_N_ELEMENTS (media_action_entries);
+	tpage->filter_func = media_info_page_filter;
 
 	return ipage;
 }
@@ -1180,6 +1495,31 @@ static GtkActionEntry links_action_entries[] =
 	  G_CALLBACK (links_copy_link_address_cb) },
 };
 
+static void
+links_drag_data_get_cb (GtkWidget *widget,
+                        GdkDragContext *context,
+                        GtkSelectionData *selection_data,
+                        guint info,
+                        guint32 time,
+                        LinksInfoPage *page)
+{
+	char *address;
+
+	g_assert (widget != NULL);
+	g_return_if_fail (context != NULL);
+
+	address = links_get_selected_link_url (page);
+	if (address == NULL) return;
+
+	gtk_selection_data_set (selection_data,
+			        selection_data->target,
+			        8,
+			        address,
+			        strlen(address));
+
+	g_free (address);
+}
+
 static void
 links_info_page_construct (InfoPage *ipage)
 {
@@ -1205,6 +1545,21 @@ links_info_page_construct (InfoPage *ipage)
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
+	/* Allow the tree to be a source for dnd */
+	gtk_tree_view_enable_model_drag_source (treeview, GDK_BUTTON1_MASK,
+                                                drag_targets, n_drag_targets,
+                                                GDK_ACTION_COPY);
+	g_signal_connect (G_OBJECT(treeview),
+                          "drag_data_get",
+                          G_CALLBACK(links_drag_data_get_cb),
+                          ipage);
+
+	/* Be careful, default handler forces dnd icon to be an image of the row ! */ 
+	g_signal_connect_after (G_OBJECT(treeview),
+                                "drag_begin",
+                                G_CALLBACK(page_info_drag_begin_cb),
+                                NULL);
+
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (treeview,
 						     COL_LINK_URL,
@@ -1258,7 +1613,7 @@ links_info_page_fill (InfoPage *ipage)
 	GtkTreeIter iter;
 	GList *links, *l;
 
-	links = mozilla_get_links (dialog->priv->embed);
+	links = dialog->priv->page_info->links;
 
 	for (l = links; l != NULL; l = l->next)
 	{
@@ -1271,9 +1626,6 @@ links_info_page_fill (InfoPage *ipage)
 				    COL_LINK_REL, link->rel,
 				    -1);
 	}
-
-	g_list_foreach (links, (GFunc) mozilla_free_embed_page_link, NULL);
-	g_list_free (links);
 }
 
 static InfoPage *
@@ -1393,7 +1745,7 @@ forms_info_page_fill (InfoPage *ipage)
 	GtkTreeIter iter;
 	GList *forms, *l;
 
-	forms = mozilla_get_forms (dialog->priv->embed);
+	forms = dialog->priv->page_info->forms;
 
 	for (l = forms; l != NULL; l = l->next)
 	{
@@ -1406,9 +1758,6 @@ forms_info_page_fill (InfoPage *ipage)
 				    COL_FORM_ACTION, form->action,
 				    -1);
 	}
-
-	g_list_foreach (forms, (GFunc) mozilla_free_embed_page_form, NULL);
-	g_list_free (forms);
 }
 
 static InfoPage *
@@ -1437,7 +1786,7 @@ page_info_dialog_init (PageInfoDialog *dialog)
 	dialog->priv = PAGE_INFO_DIALOG_GET_PRIVATE (dialog);
 
 	dialog->priv->pages[GENERAL_PAGE] = general_info_page_new (dialog);
-	dialog->priv->pages[IMAGES_PAGE] = images_info_page_new (dialog);
+	dialog->priv->pages[MEDIA_PAGE] = media_info_page_new (dialog);
 	dialog->priv->pages[LINKS_PAGE] = links_info_page_new (dialog);
 	dialog->priv->pages[FORMS_PAGE] = forms_info_page_new (dialog);
 
@@ -1479,6 +1828,9 @@ page_info_dialog_constructor (GType type,
 
 	dialog->priv->dialog = ephy_dialog_get_control (edialog, properties[PROP_DIALOG].id);
 
+	gtk_window_set_icon_name (GTK_WINDOW (dialog->priv->dialog),
+				  GTK_STOCK_PROPERTIES);
+
 	dialog->priv->manager = gtk_ui_manager_new ();
 	dialog->priv->action_group = gtk_action_group_new ("PageInfoContextActions");
 	gtk_action_group_set_translation_domain (dialog->priv->action_group,
@@ -1501,6 +1853,9 @@ page_info_dialog_constructor (GType type,
 		g_error_free (error);
 	}
 
+	dialog->priv->page_info = mozilla_get_page_info (dialog->priv->embed);
+	g_return_val_if_fail (dialog->priv->page_info != NULL, object);
+
 	for (i = GENERAL_PAGE; i < LAST_PAGE; i++)
 	{
 		page = dialog->priv->pages[i];
@@ -1526,6 +1881,7 @@ page_info_dialog_finalize (GObject *object)
 
 	LOG ("PageInfoDialog finalizing")
 
+	mozilla_free_embed_page_info (dialog->priv->page_info);
 	for (i = GENERAL_PAGE; i < LAST_PAGE; i++)
 	{
 		g_free (dialog->priv->pages[i]);
@@ -1606,3 +1962,4 @@ page_info_dialog_new (EphyWindow *window,
 			     "embed", embed,
 			     NULL);
 }
+
