@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,18 +16,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: sample.c,v 1.3 2004/12/08 15:33:52 chpe Exp $
+ *  $Id: sample.c,v 1.4 2004/12/30 22:42:23 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-sample-extension.h"
 #include "ephy-debug.h"
 
-#include <gmodule.h>
 #include <glib/gi18n-lib.h>
+#include <gmodule.h>
 
 G_MODULE_EXPORT GType register_module (GTypeModule *module);
 
