@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id$
+ * $Id: ephy-sidebar-embed.c,v 1.6 2005/01/23 23:12:11 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-sidebar-embed.h"
 #include "sidebar-commands.h"
@@ -170,15 +168,15 @@ show_context_menu (EphySidebarEmbed *sbembed,
 {
 	GtkUIManager *manager;
 	GtkAction *action;
-	EmbedEventContext context;
+	EphyEmbedEventContext context;
 	const char *popup;
 	const GValue *value;
 	gboolean framed, has_background, can_open_in_new;
 	GtkWidget *widget;
-	EphyEmbedEventType type;
 	EphyWindow *window = sbembed->priv->window;
 	gboolean hide_edit_actions = TRUE;
 	gboolean can_copy, can_cut, can_paste;
+	guint button;
 
 	ephy_embed_event_get_property (event, "framed_page", &value);
 	framed = g_value_get_int (value);
@@ -190,20 +188,29 @@ show_context_menu (EphySidebarEmbed *sbembed,
 
 	LOG ("show_embed_popup context %x", context)
 
-	if ((context & EMBED_CONTEXT_LINK) &&
-	    (context & EMBED_CONTEXT_IMAGE))
+	if ((context & EPHY_EMBED_CONTEXT_EMAIL_LINK) &&
+	    (context & EPHY_EMBED_CONTEXT_IMAGE))
+	{
+		popup = "/EphyImageEmailLinkPopup";
+	}
+	else if (context & EPHY_EMBED_CONTEXT_EMAIL_LINK)
+	{
+		popup = "/EphyEmailLinkPopup";
+	}
+	else if ((context & EPHY_EMBED_CONTEXT_LINK) &&
+		 (context & EPHY_EMBED_CONTEXT_IMAGE))
 	{
 		popup = "/EphySidebarImageLinkPopup";
 	}
-	else if (context & EMBED_CONTEXT_LINK)
+	else if (context & EPHY_EMBED_CONTEXT_LINK)
 	{
 		popup = "/EphySidebarLinkPopup";
 	}
-	else if (context & EMBED_CONTEXT_IMAGE)
+	else if (context & EPHY_EMBED_CONTEXT_IMAGE)
 	{
 		popup = "/EphySidebarImagePopup";
 	}
-	else if (context & EMBED_CONTEXT_INPUT)
+	else if (context & EPHY_EMBED_CONTEXT_INPUT)
 	{
 		popup = "/EphyInputPopup";
 		hide_edit_actions = FALSE;
@@ -214,7 +221,7 @@ show_context_menu (EphySidebarEmbed *sbembed,
 				 "/EphySidebarDocumentPopup";
 	}
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 	action = gtk_ui_manager_get_action (manager, "/EphySidebarDocumentPopup/SaveBackgroundAsDP");
 	g_object_set (action, "sensitive", has_background,
 			      "visible", has_background, NULL);
@@ -243,17 +250,18 @@ show_context_menu (EphySidebarEmbed *sbembed,
 	g_signal_connect (widget, "hide",
 			  G_CALLBACK (hide_embed_popup_cb), manager);
 
-	type = ephy_embed_event_get_event_type (event);
-	if (type == EPHY_EMBED_EVENT_KEY)
+	button = ephy_embed_event_get_button (event);
+	if (button == 0)
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
 				popup_menu_at_coords, event, 2,
 				gtk_get_current_event_time ());
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (widget), FALSE);
 	}
 	else
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
-				NULL, NULL, 2,
+				NULL, NULL, button,
 				gtk_get_current_event_time ());
 	}
 }
@@ -282,7 +290,7 @@ save_property_url (EphyEmbed *embed,
 	location = g_value_get_string (value);
 
 	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_flags (persist, 0);
@@ -299,9 +307,8 @@ embed_mouse_click_cb (EphyEmbed *embed,
 		      EphyEmbedEvent *event,
 		      EphySidebarEmbed *sbembed)
 {
-	EphyEmbedEventType type;
-	EmbedEventContext context;
-	guint modifier;
+	EphyEmbedEventContext context;
+	guint button, modifier;
 	gboolean handled = TRUE;
 	gboolean with_control, with_shift, is_left_click, is_middle_click;
 	gboolean is_link, is_image, is_middle_clickable;
@@ -310,24 +317,24 @@ embed_mouse_click_cb (EphyEmbed *embed,
 
 	g_return_val_if_fail (EPHY_IS_EMBED_EVENT(event), FALSE);
 
-	type = ephy_embed_event_get_event_type (event);
+	button = ephy_embed_event_get_button (event);
 	context = ephy_embed_event_get_context (event);
 	modifier = ephy_embed_event_get_modifier (event);
 
 	LOG ("ephy_sidebar_mouse_click_cb: type %d, context %x, modifier %x",
-	     type, context, modifier)
+	     button, context, modifier)
 
 	with_control = (modifier & GDK_CONTROL_MASK) != 0;
 	with_shift = (modifier & GDK_SHIFT_MASK) != 0;
-	is_left_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON1);
-	is_middle_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2);
+	is_left_click = (button == 1);
+	is_middle_click = (button == 2);
 
-	is_link = (context & EMBED_CONTEXT_LINK) != 0;
-	is_image = (context & EMBED_CONTEXT_IMAGE) != 0;
-	is_middle_clickable = !((context & EMBED_CONTEXT_LINK)
-				|| (context & EMBED_CONTEXT_INPUT)
-				|| (context & EMBED_CONTEXT_EMAIL_LINK));
-	is_input = (context & EMBED_CONTEXT_INPUT) != 0;
+	is_link = (context & EPHY_EMBED_CONTEXT_LINK) != 0;
+	is_image = (context & EPHY_EMBED_CONTEXT_IMAGE) != 0;
+	is_middle_clickable = !((context & EPHY_EMBED_CONTEXT_LINK)
+				|| (context & EPHY_EMBED_CONTEXT_INPUT)
+				|| (context & EPHY_EMBED_CONTEXT_EMAIL_LINK));
+	is_input = (context & EPHY_EMBED_CONTEXT_INPUT) != 0;
 
 	ephy_embed_event_get_property (event, "link_target", &targetValue);
 
@@ -403,7 +410,7 @@ ephy_sidebar_embed_create_embed (EphySidebarEmbed *sbembed)
 {
 	EphyEmbed *embed;
 
-	embed = EPHY_EMBED(ephy_embed_factory_new_object ("EphyEmbed"));
+	embed = EPHY_EMBED(ephy_embed_factory_new_object (EPHY_TYPE_EMBED));
 	
 	gtk_container_add (GTK_CONTAINER (sbembed), GTK_WIDGET (embed));
 	gtk_widget_show (GTK_WIDGET (embed));
@@ -464,7 +471,7 @@ ephy_sidebar_embed_set_window (EphySidebarEmbed *sbembed,
 
 	sbembed->priv->window = window;
 
-	manager = GTK_UI_MANAGER (window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (window));
 
 	action_group = sbembed->priv->action_group =
 		gtk_action_group_new ("SidebarContextMenuActions");
@@ -536,7 +543,7 @@ ephy_sidebar_embed_finalize (GObject *object)
 
 	g_free (sbembed->priv->url);
 
-	manager = GTK_UI_MANAGER (sbembed->priv->window->ui_merge);
+	manager = GTK_UI_MANAGER (ephy_window_get_ui_manager (sbembed->priv->window));
 
 	if (sbembed->priv->ui_id != 0)
 	{
